
"use strict";

const OFFSCREEN_DOCUMENT_PATH = 'offscreen.html';

let creatingOffscreenDocument;

async function hasOffscreenDocument() {
  const matchedClients = await clients.matchAll();
  return matchedClients.some((client) => client.url.endsWith(OFFSCREEN_DOCUMENT_PATH));
}

async function setupOffscreenDocument() {
  if (await hasOffscreenDocument()) return;

  if (creatingOffscreenDocument) {
    await creatingOffscreenDocument;
  } else {
    creatingOffscreenDocument = chrome.offscreen.createDocument({
      url: OFFSCREEN_DOCUMENT_PATH,
      reasons: [chrome.offscreen.Reason.DOM_SCRAPING],
      justification: 'Firebase Authentication'
    });
    await creatingOffscreenDocument;
    creatingOffscreenDocument = null;
  }
}

async function getAuthFromOffscreen() {
  await setupOffscreenDocument();
  return new Promise((resolve, reject) => {
    chrome.runtime.sendMessage({ action: 'signIn', target: 'offscreen' }, (response) => {
      if (chrome.runtime.lastError) {
        reject(chrome.runtime.lastError);
      } else {
        resolve(response);
      }
    });
  });
}

async function doLogoutInOffscreen() {
  await setupOffscreenDocument();
  return new Promise((resolve, reject) => {
    chrome.runtime.sendMessage({ action: 'signOut', target: 'offscreen' }, (response) => {
      if (chrome.runtime.lastError) {
        reject(chrome.runtime.lastError);
      } else {
        resolve(response);
      }
    });
  });
}

const doRefreshIdToken = async (token) => {
  await setupOffscreenDocument();
  return new Promise((resolve, reject) => {
    chrome.runtime.sendMessage({ action: 'refreshIdToken', target: 'offscreen', data: token }, (response) => {
      if (chrome.runtime.lastError) {
        reject(chrome.runtime.lastError);
      } else {
        resolve(response);
      }
    });
  });
}

chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
  const { action, data } = message;
  if (action === 'begin-process') {
    chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
      var _a;
      if ((_a = tabs[0]) === null || _a === void 0 ? void 0 : _a.id) {
        // Enviar mensaje al content script de la pestaña activa
        chrome.tabs.sendMessage(tabs[0].id, { action: action, data: data }, (response) => {
          console.log("Respuesta del content script:", response);
          sendResponse(response);
        });
      }
    });
  }

  else if (action === 'signIn') {
    getAuthFromOffscreen()
      .then(data => {
        const parsedData = JSON.parse(data)
        if (parsedData.idToken) {
          chrome.storage.session.set({ idToken: parsedData.idToken });
          chrome.storage.session.set({ refreshToken: parsedData.refreshToken });
          sendResponse({ user: parsedData });
        }
      })
      .catch(error => {
        sendResponse({ error: error.message });
      });
    return true; // Indicates we will send a response asynchronously

  } else if (message.action === 'signOut') {
    doLogoutInOffscreen()
      .then((data) => {
        const parsedData = JSON.parse(data)
        chrome.storage.local.remove('user');
        chrome.storage.session.remove('idToken');
        chrome.storage.session.remove('refreshToken');
        sendResponse(parsedData);
      }).catch(error => {
        console.error('Logout error:', error);
      })

    // Indicates we will send a response asynchronously
    return true;

  } else if (message.action === 'refreshIdToken') {
    doRefreshIdToken(data)
      .then((data) => {
        const parsedData = JSON.parse(data)
        sendResponse(parsedData);
      }).catch(error => {
        console.error('Logout error:', error);
      })

    return true;
  }
});


